<template>
  <div
    ref="container"
    class="call-container"
    :class="[
      participant && participant.length > 1 && rightPanelVisible ? 'call-container-show-right-panel' : '',
      yardmanType === '2' && memberList.length < 2 ? 'call-container-remote-only' : '',
      isFullscreen ? 'call-container-full' : '',
    ]"
    v-show="dialling"
    v-drag="{ drag: 'drag-header' }"
  >
    <!-- 头部 -->
    <div id="drag-header" class="header" @dblclick="handleScreen">
      <div class="timeTik">调度时长 {{ timeTikFormat }}</div>
      <div class="item time">{{ currentDatetime }}</div>
      <i class="item icon" :class="isFullscreen ? 'narrow' : 'fullscreen'" @click="handleScreen"></i>
      <i class="item icon close" @click.prevent="handleLeave"></i>
    </div>
    <!-- 音视频区域 大于等于两个参会人：宫格布局、小于2个参会人：1对1布局；1对1布局时区分是否视频上拉 remote-only 、C2C -->
    <div
      class="container"
      :class="[
        memberList.length >= 2 ? 'container-' + layout : '',
        memberList.length < 2 && yardmanType === '2' ? 'container-remote-only' : '',
        memberList.length < 2 && yardmanType === '1' ? 'container-C2C' : '',
      ]"
    >
      <!-- 本地流播放区域 -->
      <div v-show="yardmanType === '1'" class="stream-content">
        <el-avatar v-show="type === 'audio' || isMutedVideo" :size="60">
          <div class="avatar">{{ currentUserProfile.nick }}</div>
        </el-avatar>
        <div class="info">
          <span class="name">{{ currentUserProfile.nick }}</span>
          <i class="micro-status" :class="'micro-status-' + (isMutedAudio ? 'off' : 'on')"></i>
        </div>
        <video id="local-video" style="width: 100%; height: 100%; object-fit: fill" autoplay=""></video>
      </div>
      <!-- 远端流区域 -->
      <div
        v-for="item in memberListSort"
        :key="item.dispatchUid"
        :id="item.dispatchUid"
        class="stream-content"
        :class="[
          type === 'audio' ? 'stream-content-audio' : '',
          item.state ? 'stream-content-' + item.state : 'stream-content-CONNECTING',
        ]"
      >
        <el-avatar v-if="type === 'audio'" v-show="item.state === 'CONNECTED'" :size="60">
          <div class="avatar">{{ item.name }}</div>
        </el-avatar>
        <div class="info">
          <span class="name">{{ item.name }}</span>
          <!-- 中诺不支持，先隐藏 -->
          <!--          <i
            class="micro-status"
            :class="
              'micro-status-' + (remoteStreamList.find((x) => x.getUserId() === item.tencentUserId)?.hasAudio() ? 'on' : 'off')
            "
          ></i>-->
          <i v-if="item.handUp" class="icon-hand-up" @click="handup(item.dispatchUid)"></i>
        </div>
        <video
          :id="'remote-video-' + item.dispatchUid"
          :muted="mute"
          autoplay=""
          class="remote_video"
          :itemIndex="memberList.length"
          style="width: 100%; height: 100%; object-fit: fill"
        ></video>
      </div>
    </div>
    <!-- 参与人列表 -->
    <participated-list
      v-if="participant && participant.length > 1"
      :visible.sync="rightPanelVisible"
      @showAddMember="showAddUsers = true"
      @muteAudio="handleMuteAudio"
      @muteVideo="handleMuteVideo"
      @kickOut="kickOut"
      @recall="recall"
      @handup="handup"
      @memberMute="memberMute"
      @memberUnmute="memberUnmute"
      @muteAll="muteAll"
      @unMuteAll="unMuteAll"
    ></participated-list>
    <!-- 底部 -->
    <div class="footer">
      <div :class="!isMutedAudio ? 'micOn' : 'micOff'" @click="handleMuteAudio">
        <i class="icon"></i>
        <span class="name">麦克风</span>
      </div>
      <div
        v-if="type === 'video' && yardmanType === '1'"
        :class="!isMutedVideo ? 'videoOn' : 'videoOff'"
        @click="handleMuteVideo"
      >
        <i class="icon"></i>
        <span class="name">摄像头</span>
      </div>
      <div class="refuse" @click="handleLeave">
        <i class="icon"></i>
        <span class="name">结束</span>
      </div>
      <div v-if="memberList.length <= 9" class="layout" @click="switchLayout">
        <i class="icon"></i>
        <span class="name">切换布局</span>
        <div v-show="showSwitchLayout" class="layout-picker">
          <div class="layout-type">
            <div class="layout-type-img layout-type-img-3"></div>
            <el-radio v-model="layout" :label="3" :disabled="memberList.length >= 3">三宫格</el-radio>
          </div>
          <div class="layout-type">
            <div class="layout-type-img layout-type-img-6"></div>
            <el-radio v-model="layout" :label="6" :disabled="memberList.length >= 6">六宫格</el-radio>
          </div>
          <div class="layout-type">
            <div class="layout-type-img layout-type-img-9"></div>
            <el-radio v-model="layout" :label="9">九宫格</el-radio>
          </div>
        </div>
      </div>
      <div
        v-if="memberList.length >= 2"
        class="members"
        :class="rightPanelVisible ? 'members-active' : ''"
        @click="setRightPanelVisible"
      >
        <i class="icon"></i>
        <span class="name">成员</span>
      </div>
    </div>
    <!-- 添加成员弹框 -->
    <un-participated-list :visible.sync="showAddUsers" @addMembers="addMembers"></un-participated-list>
  </div>
</template>

<script>
/**
 * 音视频的逻辑在mixins:mqtt里处理就好了
 * 这里只处理UI逻辑
 */
import mqtt from "@/components/mixins/mqtt";
import screenfull from "screenfull";
import _ from "lodash";
import dayjs from "dayjs";
import "dayjs/locale/zh-cn";
import weekday from "dayjs/plugin/weekday";
import participatedList from "@/components/mqttClient/participatedList.vue";
import UnParticipatedList from "@/components/mqttClient/unParticipatedList.vue";
import { mapMutations, mapState } from "vuex";
import { bus } from "@/components/mqttClient/utils/bus";
dayjs.locale("zh-cn"); // 本地化
dayjs.extend(weekday); // 第一天为周一

export default {
  name: "mqttClient",
  mixins: [mqtt],
  components: { UnParticipatedList, participatedList },
  props: {},
  data() {
    return {
      dialling: false, // 是否拨打电话中
      calling: false, // 是否通话中
      isDialled: false, // 是否被呼叫
      rightPanelVisible: true,
      isFullscreen: false,
      timer: null, // 当前时间定时器
      currentDatetime: dayjs().format("YYYY-MM-DD dddd HH:mm:ss"), // 当前时间
      timeTik: 0,
      timeTikFormat: "",
      timingTimer: null,
      openTime: null,
      allMembersTimer: null,
      connectingLimit: 60, // 拨号时长上限 单位秒 默认 1分钟（60秒）
      showAddUsers: false,
      layout: 3,
      showSwitchLayout: false,
      mute: false,
      micUserId: "",
      currentPage: 0,
    };
  },
  computed: {
    ...mapState({
      account: ({ gwsd }) => gwsd.account,
      memberList: ({ gwsd }) => gwsd.memberList,
      currentUserProfile: ({ user }) => user.currentUserProfile,
      participant: ({ gwsd }) => gwsd.participant,
      yardmanType: ({ gwsd }) => gwsd.yardman.type,
      type: ({ gwsd }) => gwsd.yardman.callType,
      audio: ({ gwsd }) => gwsd.yardman.audio,
      video: ({ gwsd }) => gwsd.yardman.video,
      isMutedAudio: ({ gwsd }) => gwsd.yardman.isMutedAudio,
      isMutedVideo: ({ gwsd }) => gwsd.yardman.isMutedVideo,
    }),
    memberListSort() {
      return [...this.memberList].sort((a, b) => {
        return a?.state?.localeCompare(b?.state);
      });
    },
    memberListMoreSort() {
      return function (num) {
        return [...this.memberList]
          .slice((num - 1) * (num <= 2 ? 8 : 9), (num <= 2 ? 8 : 9) * num)
          .filter((x) => x !== null && x !== undefined)
          .sort((a, b) => a?.state?.localeCompare(b?.state));
      };
    },
    containerTranslate() {
      return {
        transform: `translate(${-this.currentPage * 100}%,0)`,
      };
    },
  },
  watch: {
    memberList: {
      handler(value) {
        // console.log("memberList", value);
        const offlineMembers = [...value].filter((x) => x.state === "DISCONNECTED").length;
        // console.log("offline members", offlineMembers, value.length);
        if (offlineMembers >= value.length && this.dialling) {
          this.handleLeave();
        }
      },
      deep: true,
      immediate: true,
    },
  },
  methods: {
    ...mapMutations(["SET_YARDMAN_AUDIO_MUTED", "SET_YARDMAN_VIDEO_MUTED"]),
    handleScreen() {
      if (screenfull.isEnabled) {
        // console.log("screen",this.$refs.container)
        screenfull.toggle(this.$refs.container);
        this.isFullscreen = !this.isFullscreen;
        if (!this.isFullscreen) {
          this.$refs.container.style.top = "0";
          this.$refs.container.style.left = "50%";
          this.$refs.container.style.transform = "translate(-50%, 0)";
        }
      }
    },
    setTimer() {
      this.timer = setTimeout(() => {
        this.currentDatetime = dayjs().format("YYYY-MM-DD dddd HH:mm:ss");
        this.setTimer();
      }, 1000);
    },
    setTimingTimer() {
      this.timingTimer = setTimeout(() => {
        if (this.dialling) {
          this.timeTik = dayjs().diff(this.openTime) || 0;
          this.timeTikFormat = dayjs(this.timeTik).format("mm:ss");
        }
        this.setTimingTimer();
      }, 1000);
    },
    setAllMembersTimer() {
      this.allMembersTimer = setInterval(() => {
        const memberList = this.memberList.map((member) => {
          const hasKey = Object.prototype.hasOwnProperty.call(member, "timer");
          const beKickOut = member.beKickOut;
          if (!hasKey || beKickOut || member.state === "CONNECTED") {
            member.timer = 0;
          } else {
            if (member.timer < this.connectingLimit && member.state === "CONNECTING") {
              member.timer++;
              member.timeout = false;
            } else {
              // 进入了超时判断
              if (member.state !== "DISCONNECTED") {
                member.timer = 0;
                member.timeout = true;
                member.state = "DISCONNECTED";
              }
            }
          }
          return member;
        });
        this.SET_MEMBER_LIST(memberList); // 更新被调度人数组
        // console.log(
        //   "%c群聊定时器 %c成员列表",
        //   "background: #E6A23C; color: #fff; padding:2px",
        //   "background: #00cc00; color: #fff; padding:2px",
        //   this.memberList
        // );
      }, 1000);
    },
    clearTimer() {
      if (this.timer) {
        clearTimeout(this.timer);
        this.timer = null;
      }
      if (this.timingTimer) {
        clearTimeout(this.timingTimer);
        this.timingTimer = null;
      }
      if (this.allMembersTimer) {
        clearInterval(this.allMembersTimer);
        this.allMembersTimer = null;
      }
      this.timeTikFormat = "";
    },
    handleLeave: _.debounce(function () {
      this.clearTimer();
      this.leave().then(() => {
        this.dialling = false;
        this.SET_YARDMAN_VIDEO_MUTED(false);
        // this.isMutedAudio = false;
        this.SET_YARDMAN_AUDIO_MUTED(false);
        this.rightPanelVisible = true;
        this.rightPanelActiveName = "1";
        // this.$bus.$off("mqtt-call", this.videoCalling);
      });
    }, 500),
    // 发起通话
    async videoCalling() {
      this.layout = 3;
      if (this.memberList.length >= 3) {
        this.layout = 6;
        if (this.memberList.length >= 6) {
          this.layout = 9;
        }
      }
      console.log("videoCalling---", this.account);
      this.initClient(this.account).then(() => {
        try {
          this.dialling = true; // 显示窗口
          this.initListener();
        } catch (e) {
          console.log("videoCalling error", e);
        }
      });
    },
    initListener(init = true) {
      // 连接音视频服务器成功
      this.client.on("connect", () => {
        if (!this.dialling) return;
        // 开始会话时开始计时
        this.openTime = dayjs();
        this.setTimingTimer();
        // 分别对所有参会人计时
        this.setAllMembersTimer();
        // 打开本地流
        this.initLocalStream({
          audio: this.audio,
          video: this.video,
        });
      });
      // 发布本地流成功
      this.client.on("localVideoOpen", ({ userId, mediaId }) => {
        console.log("发布本地流成功", userId, mediaId);
        // 邀请入会
        console.log(
          "%c刚建立会议自动邀请 %c成员列表",
          "background: #E6A23C; color: #fff; padding:2px",
          "background: #00cc00; color: #fff; padding:2px",
          this.memberList
        );
        if (init)
          this.inviteRemote(this.memberList, {
            audioonly: this.type === "audio", // 音频调度
          });
      });
      // 发布本地流失败
      this.client.on("localVideoError", (err) => {
        this.$message.error(err);
      });
    },
    pagePre() {
      this.currentPage--;
      if (this.currentPage < 0) {
        this.currentPage = Math.ceil(this.memberList.length / 9) - 1;
      }
    },
    pageNext() {
      this.currentPage++;
      if (this.currentPage > Math.ceil((this.memberList.length + 1) / 9) - 1) {
        this.currentPage = 0;
      }
    },
    // 处理本地禁麦、取消禁麦
    handleMuteAudio: _.debounce(function () {
      if (this.isMutedAudio) {
        this.unmuteAudio();
      } else {
        this.muteAudio();
      }
    }, 500),
    handleMuteVideo: _.debounce(function () {
      if (this.isMutedVideo) {
        this.unmuteVideo();
      } else {
        this.muteVideo();
      }
    }, 500),
    switchLayout: _.debounce(function () {
      this.showSwitchLayout = !this.showSwitchLayout;
    }, 500),
    setRightPanelVisible: _.debounce(function () {
      this.rightPanelVisible = !this.rightPanelVisible;
    }, 500),
    // 新增成员后回调
    addMembers(memberList) {
      this.inviteRemote(memberList, {
        audioonly: this.type === "audio", // 音频调度或
      });
      if (this.memberList.length >= 3) {
        this.layout = 6;
        if (this.memberList.length >= 6) {
          this.layout = 9;
        }
      }
    },
    memberMute(id) {
      this.muteRemoteAudio(id);
    },
    memberUnmute(id) {
      this.unmuteRemoteAudio(id);
    },
    muteAll() {
      this.memberList.forEach((item) => {
        this.muteRemoteAudio(item.dispatchUid);
      });
    },
    unMuteAll() {
      this.memberList.forEach((item) => {
        this.unmuteRemoteAudio(item.dispatchUid);
      });
    },
    kickOut(id) {
      this.kickOutRemote(id);
    },
    recall(dispatchUid) {
      this.inviteRemote([{ dispatchUid }], {
        audioonly: this.type === "audio", // 音频调度
      });
    },
    handup(dispatchUid) {
      this.handUpRemote(dispatchUid);
    },
  },
  created() {
    this.setTimer();
  },
  mounted() {
    this.$bus.$on("mqtt-call", this.videoCalling); // 发起通话
    // 远端流加入
    bus.$on("pull_video_message_new_remote", (ack) => {
      console.log("pull_video_message_new_remote", ack);
      if (ack.length > 0) {
        this.pullVideoFromServer(ack[0].id);
      }
    });
    // 监听远端离开
    bus.$on("remote_HangUp", (ack) => {
      console.log("监听远端离开", ack);
      // 这里ack里面是远端离开的uid 然后你可以写她离开的逻辑
      this.pullVideoLeave(ack);
    });
  },
  beforeDestroy() {
    this.$bus.$off("mqtt-call", this.videoCalling);
    bus.$off("pull_video_message_new_remote");
    bus.$off("remote_HangUp");
  },
  destroyed() {
    this.clearTimer();
  },
};
</script>

<style lang="scss" scoped>
@import "~@/styles/rtc-client.scss";
</style>
